/* Flex - Scomp Implementation
* Uses VGA to Display Data 
* PB1 is clock for Mips
* PB2 is synchronous reset for Mips
*/
module top_flex(clock_25Mhz, pb1,pb2,
	red_out,green_out,blue_out,horiz_sync,vert_sync);
input clock_25Mhz, pb1, pb2;
// VGA display
output red_out,green_out,blue_out;
output horiz_sync, vert_sync;
wire [9:0] pixel_row, pixel_col;

wire red_in, green_in, blue_in;
wire pb1_debounced;

// declare signals used to connect VHDL SCOMP components
   wire [7:0] PC;
   wire [7:0] MAR;
   wire [15:0] AC;
   wire [15:0] MDR;
   wire [15:0] IR;
   wire [7:0] state;
   wire clock, reset;
   
   // instantiate simple computer
   scomp2 computer(clock, reset, PC, AC, MAR, MDR, IR,state);

reg [5:0] char_address;
wire [5:0] row_address, col_address;
wire rom_mux_output;
reg rev_video;

parameter space = 6'o40;

reg [5:0] format_address, format_data;

always begin
case (format_address)
 4: format_data <= space;
 5: format_data <= 6'o25; // U
 6: format_data <= 6'o20; // P
 7: format_data <= 6'o61; // 1

 8: format_data <= 6'o20; // P
 9: format_data <= 6'o03; // C
10: format_data <= space;
11: format_data <= space;

12: format_data <= 6'o11; // I
13: format_data <= 6'o16; // N
14: format_data <= 6'o23; // S
15: format_data <= 6'o24; // T

16: format_data <= 6'o01; // A
17: format_data <= 6'o03; // C
18: format_data <= space;
19: format_data <= space;

20: format_data <= 6'o15; // M
21: format_data <= 6'o04; // D
22: format_data <= 6'o22; // R
23: format_data <= space;

24: format_data <= 6'o15; // M
25: format_data <= 6'o01; // A
26: format_data <= 6'o22; // R
27: format_data <= space;

28: format_data <= 6'o22; // R
29: format_data <= 6'o17; // O
30: format_data <= 6'o25; // U
31: format_data <= 6'o24; // T

32: format_data <= 6'o03; // C
33: format_data <= 6'o14; // L
34: format_data <= 6'o13; // K
35: format_data <= space;

36: format_data <= 6'o22; // R
37: format_data <= 6'o23; // S
38: format_data <= 6'o24; // T
39: format_data <= space;

default: format_data <= space;
endcase
end

	// instantiate vga_sync
	vga_sync model1(clock_25Mhz, red_in, green_in, blue_in, red_out, green_out, blue_out, horiz_sync, vert_sync,pixel_row,pixel_col);
	

	// Colors for pixel data on video signal
	// use video_rom for pixel color data
	// rev_video will reverse video pixels
	assign red_in = ~(rom_mux_output^rev_video);
	assign green_in = ~(rom_mux_output^rev_video);
	assign blue_in = 1'b1;

	// instantiate character_rom
	char_rom model2(char_address, pixel_row[3:1], pixel_col[3:1], rom_mux_output);
/*
   MSD: dec_7seg
		PORT MAP(hex_digit => MSB,
				segment_a => MSB_a, segment_b => MSB_b, segment_c => MSB_c, 
				segment_d => MSB_d,	segment_e => MSB_e, segment_f => MSB_f, 
				segment_g => MSB_g );

   LSD: dec_7seg
		PORT MAP(hex_digit => LSB,
				segment_a => LSB_a, segment_b => LSB_b, segment_c => LSB_c, 
				segment_d => LSB_d,	segment_e => LSB_e, segment_f => LSB_f, 
				segment_g => LSB_g );
*/

/*
		-- Combine Flex Dip Switch Inputs into Switch vector
Switch <= Flex_Switch_8 & Flex_Switch_7 & Flex_Switch_6 & Flex_Switch_5 &
          Flex_Switch_4 & Flex_Switch_3 & Flex_Switch_2 & Flex_Switch_1;
*/

// current character row and column being displayed
assign row_address = pixel_row[9:4];
assign col_address = pixel_col[9:4];

// Address for Constant Character Data Format ROM
always begin
format_address[1:0] <= col_address[1:0];
format_address[5:2] <= row_address[4:1];
end


/*
-- This Process Provides Character Data for Video Display
-- by generating addresses for the Character Generator ROM
-- using the character row address and col address provided by the Video 
-- Sync process  - 40 characters by 30 lines of display area
*/

always @ (posedge clock_25Mhz)
begin

//  Reverse Video for Title at top of screen
if (row_address<=3) rev_video <= 1;
else rev_video <= 0; 

// Blank characters on edge of screen and on alternating lines
if (row_address[0]==0 | col_address < 8 | col_address > 21 ) char_address <= space;
// Constant Character Area - use data from format ROM
else if (col_address>= 8 & col_address <= 11) char_address <= format_data;
// couple of spaces and blanks on top and bottom line of display area
else if (col_address==12 | col_address==13 | row_address < 2 | row_address > 27 ) char_address <= space;
// numeric data from simulation, display values in hex
else case (row_address[4:1])
1: 
	// print "computer" on first line of data display area
	case (col_address)
	14: char_address <= 6'o03; // C
	15: char_address <= 6'o17; // O
	16: char_address <= 6'o15; // M
	17: char_address <= 6'o20; // P
	18: char_address <= 6'o25; // U
	19: char_address <= 6'o24; // T
	20: char_address <= 6'o05; // E
	21: char_address <= 6'o22; // R
	default: char_address <= space;
	endcase
2:
	// PC register
	case (col_address)
	16: char_address <= {2'b11, PC[7:4]};
	17: char_address <= {2'b11, PC[3:0]};
	default: char_address <= space;
	endcase
3:
	// INSTRUCTION register
	case (col_address)
	14: char_address <= {2'b11, IR[15:12]};
	15: char_address <= {2'b11, IR[11:8]};
	16: char_address <= {2'b11, IR[7:4] };
	17: char_address <= {2'b11, IR[3:0] };
	default: char_address <= space;
	endcase

4:
	// ACCUMULATOR register
	case (col_address)
	14: char_address <= {2'b11, AC[15:12]};
	15: char_address <= {2'b11, AC[11:8]};
	16: char_address <= {2'b11, AC[7:4] };
	17: char_address <= {2'b11, AC[3:0] };
	default: char_address <= space;
	endcase

5:
	// Memory Data Register (MDR)
	case (col_address)
	14: char_address <= {2'b11, MDR[15:12]};
	15: char_address <= {2'b11, MDR[11:8]};
	16: char_address <= {2'b11, MDR[7:4] };
	17: char_address <= {2'b11, MDR[3:0] };
	default: char_address <= space;
	endcase

6:
	// Memory Address Register (MAR)
	case (col_address)
	16: char_address <= {2'b11, MAR[7:4] };
	17: char_address <= {2'b11, MAR[3:0] };
	default: char_address <= space;
	endcase

7:
	// State
	case (col_address)
	16: char_address <= {2'b11, state[7:4] };
	17: char_address <= {2'b11, state[3:0] };
	default: char_address <= space;
	endcase

8:
	if (col_address==17) char_address <= {4'b0111, pb1_debounced, 1'b0};
	else char_address <= space;

9:
	// select up arrow or down arrow character address
	if (col_address==17) char_address <= {4'b0111, reset, 1'b0};
	else char_address <= space;

default:
	char_address <= space;
endcase
end

/*
// Values to Display in 7Seg LEDs
assign MSB_dp = ~pb1_debounced;
assign LSB_dp 	= ~reset;
assign MSB = PC[7:4];
assign LSB = PC[3:0];
*/


/* Debounce Button: Filters out mechanical bounce for around 64Ms.
*  Debounce clock uses Vert_Sync timing signal (16Ms) to save hardware
*  needed for a clock prescaler
*/

// instantiate debounced push button
debounce bnc1(pb1_debounced, ~pb1, vert_sync);
debounce bnc2(reset, ~pb2, vert_sync);

// instantiate one-shot
one_pulse pls(clock, pb1_debounced, clock_25Mhz);

endmodule


